'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.dispatchCustomEvent = dispatchCustomEvent;
exports.dispatchClonedEvent = dispatchClonedEvent;

var _objects = require('./objects.js');

var _types = require('./types.js');

var CUSTOM_EVENT_ARG_NAMES = ['bubbles', 'cancelable', 'detail'];

var DEFAULT_CUSTOM_EVENT_OPTIONS = {
    bubbles: true,
    cancelable: true,
    detail: null
};

function createCustomEvent(eventType, options) {
    var event = void 0,
        args = (0, _objects.assign)({}, DEFAULT_CUSTOM_EVENT_OPTIONS, options);
    if (typeof CustomEvent === 'function') {
        event = new CustomEvent(eventType, args);
    } else {
        event = document.createEvent('CustomEvent');
        event.initCustomEvent.apply(event, [eventType].concat(CUSTOM_EVENT_ARG_NAMES.map(function (name) {
            return args[name];
        })));
    }
    return event;
}

/**
 * Create and dispatch a custom event.
 * @param {!HTMLElement} el the element where the event will be dispatched
 * @param {!string} eventType the event type
 * @param {Object} [options] the options
 * @returns {boolean} false if at least one of the event handlers which handled this event called Event.preventDefault(). Otherwise it returns true.
 */
function dispatchCustomEvent(el, eventType, options) {
    return el.dispatchEvent(createCustomEvent(eventType, options));
}

/**
 * Create and dispatch an event based on another one.
 * The properties of the base event not found into the new one will be copied.
 * @param el the element where the new event will be dispatched
 * @param inEvt the base event
 * @returns {boolean} false if at least one of the event handlers which handled this event called Event.preventDefault(). Otherwise it returns true.
 */
function dispatchClonedEvent(el, inEvt) {
    var outEvt = createCustomEvent(inEvt.type, {
        bubbles: inEvt.bubbles,
        cancelable: inEvt.cancelable,
        view: inEvt.view,
        detail: inEvt
    });
    var keys = Object.keys(outEvt);
    Object.keys(inEvt).filter(function (k) {
        return !(0, _types.isFunction)(inEvt[k]);
    }).filter(function (k) {
        return keys.indexOf(k) < 0;
    }).forEach(function (k) {
        return outEvt[k] = inEvt[k];
    });
    return el.dispatchEvent(outEvt);
}